<?php
/**
 * @package Nouri.sh
 * @author Petar Dzhambazov
 * @version v1.0.12
 */


class Nourish2
{
    protected $campaigns;
    protected $notsigned = 'You are have no registered campaigns.';
    protected $username;
    protected $password;
    protected $title = 'eNewsletter';
    protected $templates;
    protected $basedir;
    protected $url;
    protected $label = 'Please subscribe to our newsletter';
    protected $notify = false;
    protected $show_subscribers = false;
    const NOPTIONS = 'NourishCampaigns';
    const NUSER = 'NourishUser';
    const WIDGETNOPTIONS = 'NourishWidget';
    /**
     * Constructor
     *
     * All this does is to determine where templates directory is.
     * @return void
     */
    function __construct ()
    {
        $dir = dirname(__FILE__);
        $idir = explode(DIRECTORY_SEPARATOR, $dir);
        $curdir = array_pop($idir);
        $nourishdir = array_pop($idir);
        $plurl = WP_PLUGIN_URL . '/' . $nourishdir;
        $this->url = $plurl;
        $this->basedir = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $nourishdir;
        $this->templates = $this->basedir . DIRECTORY_SEPARATOR . 'templates';
    }
    /**
     * Load
     *
     * Load plugin data. Loads options from wordpress options page.
     * On success sets private property $this->campaigns to retrieved
     * options and returns true. On failure returns false.
     *
     * @param array $args
     * @return bool
     */
    function load (array $args = array())
    {
        $widgetoptions = get_option(self::WIDGETNOPTIONS);
        if ($widgetoptions){
            $this->label = isset($widgetoptions['label'])?$widgetoptions['label']:'';
            $this->notify = isset($widgetoptions['notify'])?$widgetoptions['notify']:false;
            $this->show_subscribers = isset($widgetoptions['show'])?$widgetoptions['show']:false;
        }
        $options = get_option(self::NOPTIONS);
        if ($args && is_array($args)) {
            $updated = $this->_updateCampaigns($args);
            return $updated;
        	/*foreach ($args as $key => $value) {
        	    if (isset($options[$key])){
        		    $active = $options[$key]['active'];
        		    $options[$key] = $value;
        		    $options[$key]['active'] = $active;
        	    }
        	}
            $newoptions = $this->compareCampaigns($args);
            if ($newoptions && is_array($newoptions)) {
                $options = $this->merge($options, $newoptions);
            }*/
        }
        if ($options && is_array($options)) {
            $this->campaigns = $options;
            update_option(self::NOPTIONS, $this->campaigns);
            return true;
        }
        return false;
    }

    protected function merge($oldoptions, $newoptions){
        $result = array();
        if ($oldoptions && is_array($oldoptions)){
            $result = $oldoptions;
        }
        if ($newoptions && is_array($newoptions)){
            foreach ($newoptions as $key => $value) {
                if (isset($result[$key])){
                	$active = $result[$key]['active'];
        		    $result[$key] = $value;
        		    $result[$key]['active'] = $active;
                }else {
                	$result[$key] = $value;
                }
            }
        }
        return $result;
    }
    /**
     * Display options page
     *
     * Displays options page in Wordpress admin section.
     * Gets the campaigns and if present, gets a google chart
     * image. After that includes options template and echoes
     * the result
     *
     * @return void - result is echoed
     */
    function nourish_options ()
    {
        $campaigns = $this->campaigns();
        if ($campaigns){
            $googlechart = $this->nourish_chart($campaigns);
        }
        include_once $this->templates . '/options.phtml';
    }
    /**
     * Get google chart
     *
     * Composes an image tag string which represents google chart image.
     * Uses googleChart.php , composes data out of every campaign present
     * and passes it to googleChat class to construct an object, which returns
     * the image tag.
     *
     * @param array $campaigns
     * @return string
     */
    protected function nourish_chart($campaigns) {
        if ($campaigns){
        	require_once 'googleChart.php';
        	$data = array();
        	foreach ($campaigns as $campaign) {
        	    $label = $campaign['campaignName'];
        	    if (strlen($label) > 25){
        	        $label = wordwrap($label, 25, '[br]', false);
        	        $label = explode('[br]', $label);
        	        $label = array_shift($label);
        	        $label .= ' ... ';
        	    }
        	    $label .=  ": " . $campaign['campaignSubscribers'];
        		$data[] = array('label' => $label, 'value' => $campaign['campaignSubscribers']);
        	}
        	$options = array('cht' => 'p3', 'title' => 'eNewsletter Subscribers', 'width' => 600, 'height' => 180, 'bgcolor' => 'F1F1F1');
        	$gg = new googleChart($data, $options);
        	return $gg->getImageTag();
        }
    }
    /**
     * Include admin css
     *
     * Includes some css used in admin part, into the head part of
     * admin area.
     * @return void
     */
    function nourish_admin_styles() {
    	include $this->basedir . '/css/admin.css.html';
    }
    /**
     * Add nourish menu
     *
     * Uses wordpress hook to add an options page to wordpress main admin area
     * @return void
     */
    function nourish_menu ()
    {
        $file = __FILE__;
        add_menu_page('Nourish Campaigns', 'Nourish', 8, $file, array($this , 'nourish_options'));
    }
    /**
     * Accessior for campaigns
     *
     * Checks if there are stored campaigns in $this->canpaigns
     * if yes - returns them, if not initiates them by using
     * {@see Nourish:: () load()}
     *
     * @return array
     */
    public function campaigns ()
    {
        if (! isset($this->campaigns)) {
            $this->load();
        }
        return $this->campaigns;
    }
    /**
     * Display widget
     *
     * Displays the widget into sidebar. Accepts an array of args from
     * wordpres - such as before widget, after widget etc. common details
     * @param array $args
     * @return mixed - echoes the widget on success or returns false on failure
     */
    public function display_widget ($args)
    {
        extract($args);
        $display = (isset($_COOKIE['nourish_subscribed']))?false:true;
        if ($this->init_widget() && $display) {
            $loaded = $this->load($this->_getCampaigns('app.nouri.sh'));
            if ($loaded){
                $campaigns = $this->campaigns();
            }
            $count = 0;
            foreach ($campaigns as $campaign) {
            	$count += $campaign['active'];
            }

            if (isset($campaigns) && count($campaigns) > 0){
                ob_start();
                include_once $this->templates . '/widget.phtml';
                $result = ob_get_clean();
                echo $result;
            }
        } else {
            Nourish2::setIsSubscribed(true);
            return false;
        }
    }

    /**
     * Set cookie showing that client is subscribed, or
     * unsubscribe him by passing false
     * @param bool $subscribed
     */
    public static function setIsSubscribed($subscribed = true)
    {
        $cookieName = 'nourish_subscribed';
        $cookieValue = $subscribed ? 1 : false;
        if(function_exists('get_option')){
            $site = get_option('siteurl');
        }
        if(!isset($site)) {
            if(defined('WP_SITEURL')) {
                $site = WP_SITEURL;
            }else {
                $site = $_SERVER['HTTP_HOST'];
            }
        }
        $domain = str_ireplace('http://', '', $site);
        $period = 60 * 60 * 24 * 180;
        $time = $subscribed ? (time() + $period) : (time() - $period);
        if(!headers_sent()) {
            setcookie($cookieName, $cookieValue, $time, '/', $domain);
        }
    }

    /**
     * Add javascript to admin page
     *
     * Adds Javascript for ajax calls to admin page. SACK library
     * ships with WordPress, admin_js.phtml is custom page containing
     * javascript.
     *
     * @return void - echoes the result
     */
    function nourish_js_admin_header ()
    {
        // use JavaScript SACK library for Ajax
        wp_print_scripts(array('sack'));
        ob_start();
        include_once $this->templates . '/admin_js.phtml';
        $result = ob_get_clean();
        echo $result;
    }
    /**
     * Add javascript to User page
     *
     * Adds Javascript for ajax calls to user page. SACK library
     * ships with WordPress, user_js.phtml is custom page containing
     * javascript.
     *
     * @return void - echoes the result
     */
    function nourish_js_header ()
    {
        // use JavaScript SACK library for Ajax
        wp_print_scripts(array('sack'));
        $c = count($this->campaigns());
        ob_start();
        include_once $this->templates . '/user_js.phtml';
        $result = ob_get_clean();
        echo $result;
    }
    /**
     * AJAX remote login
     *
     * This function provides means to connect to nouri.sh main app and
     * retrieve a list with campaigns and data - camapaign id(number),
     * campaign name (string), campaign subject(string), campaign subscribers(number).
     * Uses SimpleXML to decode data. Tries to load user credentials from database on
     * failure checks if thy are passed via form. Calls a remote login page with credentials.
     * On success parses the result, compares to current data if present, if there are differences
     * they get displayed to the user and then sotred into database, if not, a message
     * 'no new campaigns' is displayed.
     * If login fails a 'login failed' message is displayed.
     * @return void
     */
    function nourish_ajax_login ()
    {
        $nourish = $this;
        if (! extension_loaded('SimpleXML')) {
            dl('SimpleXML');
        }
        if(isset($_POST['nousername']) && isset($_POST['nopassword'])){
            $username = $_POST['nousername'];
            $pass = $_POST['nopassword'];
        }else if ($nourish->loadUser()) {
            $username = $nourish->getUserName();
            $pass = $nourish->getPass();
        }
        $url = "app.nouri.sh";
        $campaigns = $this->_getCampaigns($url, $username, $pass);
        if ($campaigns) {
            if(is_array($campaigns) && count($campaigns) > 0){
                if($nourish->campaigns()) {
                    $diff = $nourish->compareCampaigns($campaigns);
                } else {
                    $nourish->load($campaigns);
                    $diff = $nourish->campaigns();
                }
                if ($diff) {
                    $text = "You have " . count($diff) . " new campaigns.<br/>\n";
                    $text .= "<ul>\n";
                    foreach ($diff as $d) {
                        $text .= "<li>$d[campaignName] - $d[campaignSubscribers] subscribers</li>\n";
                    }
                    $text .= "</ul>";
                } else {
                    $text = "<p>No new campaigns</p>";
                }
                $nourish->_updateCampaigns($campaigns);
            }else {
            	update_option(Nourish2::NOPTIONS, '');

            	$nourish->load();
            	$text = 'You have no campaigns, or campaigns have been deleted.<script type="text/javascript">window.location.reload(true)</script>';
            }
            $credentials = array('user' => $username , 'password' => $pass);
            $nourish->setUser($credentials);
            die($text);
            //    die("document.getElementById('$resultDiv').innerHTML='$text'");
        } else {
            $text = "Could not verify credentials to server.<br/>
                    Please check your user name and password and try again.";
            die($text);
            //    die("document.getElementById('$resultDiv').innerHTML='$text'");
        }
    }

    protected function _updateCampaigns($campaigns){
        if (!is_array($campaigns)){
            return false;
        }
        $oldcampaigns = $this->merge($this->campaigns(), $campaigns);
        foreach ($oldcampaigns as $id => $camp) {
        	if (!isset($campaigns[$id])){
        		unset($oldcampaigns[$id]);
        	}
        }
        $this->campaigns = $oldcampaigns;
        update_option(self::NOPTIONS, $this->campaigns);
        return true;
    }
    /**
     * Proxy to Nourish::register_widget()
     */
    function nourish_init ()
    {
        $this->register_widget();
    }
    /**
     * Proxy to Nourish::load()
     */
    public function init_widget ()
    {
        return $this->load();
    }
    /**
     * Load user
     *
     * Load user credentials for nouri.sh from options table.
     * After that stores them as protected properties. They
     * are used later to retrieve user's campaigns
     *
     * @return boolean
     */
    public function loadUser ()
    {
        $userOptions = get_option(self::NUSER);
        if ($userOptions) {
            $this->username = $userOptions['user'];
            $this->password = $userOptions['password'];
            return true;
        }
        return false;
    }
    /**
     * Get username
     *
     * Accessor retrieves username
     * @return string
     */
    public function getUserName ()
    {
        if (!isset($this->username)){
            $this->loadUser();

        }
        return $this->username;
    }
    /**
     * Get password
     *
     * Accessor retrieves password
     * @return string
     */
    public function getPass ()
    {
        if (!isset($this->password)){
            $this->loadUser();

        }
        return $this->password;
    }
    /**
     * Set user
     *
     * Sets user details (username, password)
     * into database table options. These are credentials
     * for nouri.sh used to fetch registered campaigns and
     * subscribers.
     *
     * @param array $args
     */
    public function setUser ($args)
    {
        if (is_array($args)) {
            update_option(self::NUSER, $args);
        }
    }
    /**
     * Subscribe
     *
     * This is the actual subsctibtion function, that calls the
     * REST subscription on nouri.sh
     *
     * @param  int $campaignID
     * @param  string $email
     * @return string
     */
    public static function subscribe ($campaignID, $email)
    {
        $host = "app.nouri.sh";
        $path = '/campaigns/' . $campaignID . '/subscribe.xml';
        $type = "http";
        $data = array('subscriber[email]' => $email);
        $result = self::_get($type, $host, '80', $path, $data);
        return $result;
    }
    /**
     * A function to handle fsock requests
     * @param string $type
     * @param string $host
     * @param string optional $port
     * @param string optional $path
     * @param array optional $data
     * @return string - response from conection
     */
    protected static function _get ($type, $host, $port = '80', $path = '/', array $data = array())
    {
        $_err = 'lib sockets::' . __FUNCTION__ . '(): ';
        $str = '';
        switch ($type) {
            case 'http':
                $type = '';
                break;
            case 'ssl':
                continue;
            default:
                die($_err . 'bad $type');
        }
        if (! ctype_digit($port)) {
            die($_err . 'bad port');
        }
        if (! empty($data)) {
            foreach ($data as $k => $v) {
//                $str .= urlencode($k) . '=' . urlencode($v) . '&';
                $str .= $k . '=' . $v . '&';
            }
            $str = substr($str, 0, - 1);
        }
        $fp = fsockopen($host, $port, $errno, $errstr, $timeout = 30);
        if (! $fp) {
            die($_err . $errstr . $errno);
        } else {
            // Check if magic_quotes_runtime is active
            if(get_magic_quotes_runtime())
            {
                // Deactivate
                set_magic_quotes_runtime(false);
            }
            fputs($fp, "POST $path HTTP/1.1\r\n");
            fputs($fp, "Host: $host\r\n");
            fputs($fp, "Content-type: application/x-www-form-urlencoded\r\n");
            fputs($fp, "Content-length: " . strlen($str) . "\r\n");
            fputs($fp, "Connection: close\r\n\r\n");
            fputs($fp, $str . "\r\n\r\n");
            $d = '';
            while (! feof($fp)) {
                $d .= fgets($fp, 4096);
            }
            fclose($fp);
        }
        return $d;
    }
    /**
     * Register widget
     *
     * Registers the widget options page to admin widget page.
     * Gives the user an option to choose which campaigns are available
     * for subscription.
     */
    public function register_widget ()
    {
        $campaigns = $this->campaigns();
        $id = "nourish-campaign-widget"; // Never never never translate an id
        $dims = array('width' => 350 , 'height' => 580);
        $name = __('Nouri.sh eNewsletter');
        if (count($campaigns)) {
            wp_register_sidebar_widget($id, $name, array($this , 'display_widget'));
        }
        wp_register_widget_control($id, $name, array($this , 'control_widget'), $dims);
    }
    /**
     * Display widget control
     *
     * Populates widget control page with details, and displays it.
     *
     * @param array $args
     * @return void - echoes the result
     */
    public function control_widget($args = null) {
    	$campaigns = $this->campaigns();
	    $campaigns = $this->deactivate($campaigns);
    	if (isset($_POST['nourish_activate'])){
    	    $activate = $_POST['nourish_activate'];
    	    foreach ($activate as $active => $option) {
    	    	$campaigns[$active]['active'] = $option;
    	    }
    	    $campaigns_updated = 'Active campaigns were updated.';
    	    update_option(self::NOPTIONS, $campaigns);
    	}
    	$widgetoptions = get_option(self::WIDGETNOPTIONS);
        $newwidgetoptions = array();
    	if (isset($_POST['nourish_label'])){
    	    $newwidgetoptions['label'] = htmlentities(strip_tags(trim($_POST['nourish_label'])));
    	}
    	if (isset($_POST['nourish_email_send'])){
    	    $newwidgetoptions['notify'] = true;
    	}else {
    		$newwidgetoptions['notify'] = false;
    	}

    	if (isset($_POST['nourish_show_subscribers'])){
    	    $newwidgetoptions['show'] = true;
    	}else {
    		$newwidgetoptions['show'] = false;
    	}
        if ($newwidgetoptions != $widgetoptions){
    	   $widgetoptions = $newwidgetoptions;
    	   update_option(self::WIDGETNOPTIONS, $widgetoptions);
        }
	    $this->load();
    	ob_start();
    	include $this->templates . '/control.phtml';
    	$result = ob_get_clean();
    	echo $result;
    }
    /**
     * Deactivate campaign
     *
     * Deactivates passed campaigns from being displayed for subscription
     * by setting their active property to 0. Returns deactivated campaigns.
     *
     * @param array $camps
     * @return array
     */
    protected function deactivate($camps) {
    	foreach ($camps as &$campaign) {
    		if ($campaign['active'] != 0)
    		      $campaign['active'] = 0;
    	}
    	return $camps;
    }
    /**
     * Convert campaign format
     *
     * Converts the array received from nouri.sh to the
     * format used with the plugin.
     * @param array $camp
     * @return mixed array on success, boolean false on failure
     */
    protected function convert (array $camp)
    {
        if (is_array($camp)) {
            foreach ($camp as $c) {
                $result[$c['campaignid']] = $c;
            }
            return $result;
        }
        return false;
    }
    /**
     * Compare campaigns
     *
     * Compares campaign collections to check if there are
     * new campaigns. Returns the difference as array or
     * false if no difference or if wrong parameter is passed.
     *
     * @param array $camp
     * @return mixed
     */
    public function compareCampaigns (array $camp)
    {
        /*if (! $newcamp = $this->convert($camp)) {
            return false;
        }*/
        $diff = false;
        $oldcamp = $this->campaigns();
        if ($oldcamp && is_array($oldcamp)) {
//            $diff = array_diff_assoc($camp, $oldcamp);
            $diff = $this->getDifference($camp, $oldcamp);
        } else {
            $diff = $camp;
        }
        return $diff;
    }
    /**
     * Get difference between 2 arrays
     * @param array $arrayNew
     * @param array $arrayOld
     * @return array
     */
    protected function getDifference(array $arrayNew,array $arrayOld) {
    	$diff = array();
    	foreach ($arrayNew as $key => $value) {
    		if (!isset($arrayOld[$key])){
    		    $diff[$key] = $value;
    		}
    	}
    	return $diff;
    }
    /**
     * Update campaigns
     *
     * Updates some parameters of the campaigns, localy only.
     * Also deletes camapigns, and clears all camapigns.
     *
     * @return void
     */
    public function update(){
        $clear = isset($_POST['nourish_clear'])?TRUE:FALSE;
        $deleteIds = isset($_POST['nourishIDs'])?$_POST['nourishIDs']:false;
        $names = isset($_POST['nourishNames'])?$_POST['nourishNames']:false;
        $capaigns = get_option(self::NOPTIONS);

        if ($clear){
            delete_option(self::NOPTIONS);
            return ;
        }
        if ($deleteIds){
            foreach ($deleteIds as $id) {
            	unset($capaigns[$id]);
            }
        }
        if ($names){
            foreach ($names as $key => $name) {
            	if(isset($capaigns[$key]))
            	   $capaigns[$key]['campaignName'] = $name;
            }
        }
        update_option(self::NOPTIONS, $capaigns);
    }
    /**
     *
     * @param string $url
     * @param string $user
     * @param string $pass
     * @return mixed array|false
     */
    protected function _getCampaigns($url=null, $user=null, $pass=null){
        if (null != $user){
            $username = $user;
        }else{
            $username = $this->getUserName();
        }
        if(null != $pass){
            $password = $pass;
        }else{
            $password = $this->getPass();
        }
        if(null == $url){
            $url = 'app.nouri.sh';
        }
        if (! empty($username) && ! empty($password)) {
            $url = "http://$username:$password@$url/campaigns.xml";
            $result = @file_get_contents($url);
            if ($result) {
                $xml = simplexml_load_string($result);
                foreach ($xml as $campaign) {
                    /*foreach ($campaign as $elm) {
                        $elementName = str_ireplace('-', '_', $elm->getName());
                        $value = (string)$elm;
                        $$elementName = $value;
                    }*/
                    $id = (string)$campaign->id;
                    $campaigns[$id] = array(
                        'campaignid' => $id,
                        'campaignName' => (string)$campaign->name,
                        'campaignSubject' => (string)$campaign->subject,
                        'campaignSubscribers' => (string)$campaign->{'subscribers-count'},
                        'active' => 1);
                }
                return $campaigns;
            }
            return false;
        }
        return false;
    }
}
